<?php
/**
*
* @package Chat
* @version 0.2
* @copyright (c) 2008 phpBB3.PL Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
* Minimum PHP version: PHP 5.1
*/

/**
 * Main class of the installation toolkit.
 *
 * @package Chat
**/
abstract class InstallationTools
{
	public static $schemaChangesTypes = array(
		'addColumns'       => 'add_columns',
		'changeColumns'    => 'change_columns',
		'dropColumns'      => 'drop_columns',

		'addPrimaryKeys'   => 'add_primary_keys',
		'addUniqueIndexes' => 'add_unique_index',
		'addIndexes'       => 'add_index',
		'dropIndexes'      => 'drop_keys',
	);

	public static function createDBConnection()
	{
		global $phpbb_root_path, $phpEx, $dbms, $table_prefix;

		include($phpbb_root_path . 'config.' . $phpEx);
		include($phpbb_root_path . 'includes/constants.' . $phpEx);
		include($phpbb_root_path . 'includes/db/' . $dbms . '.' . $phpEx);

		global $db;
		$db = new $sql_db();
		$db->sql_connect($dbhost, $dbuser, $dbpasswd, $dbname, $dbport, false, defined('PHPBB_DB_NEW_LINK') ? PHPBB_DB_NEW_LINK : false);

		// NOTE: trigger_error does not work here.
		$db->sql_return_on_error(true);
	}

	public static function createDBTools()
	{
		global $db;
		if (empty($db))
		{
			self::createDBConnection();
		}

		include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);

		global $dbtools;
		$dbtools = new phpbb_db_tools($db);
	}

	public function install()
	{
		global $db;
		if (empty($db))
		{
			self::createDBConnection();
		}

		if (method_exists($this, 'preInstall'))
		{
			$this->preInstall();
		}

		if (!empty($this->tables))
		{
			self::addTables($this->tables);
		}

		if (!empty($this->schemaChanges))
		{
			self::performSchemaChanges($this->schemaChanges);
		}

		if (!empty($this->insertData))
		{
			self::insertData($this->insertData);
		}

		if (!empty($this->sqlQueries))
		{
			self::performSqlQueries($this->sqlQueries);
		}

		if (!empty($this->permissionOptions))
		{
			self::addPermissions($this->permissionOptions);
		}

		if (method_exists($this, 'postInstall'))
		{
			$this->postInstall();
		}
	}

	public static function getTableName($table)
	{
		$constant = strtoupper($table) . '_TABLE';
		if (!defined($constant))
		{
			global $table_prefix;
			define($constant, $table_prefix . $table);
		}

		return constant($constant);
	}

	public static function addTables($tables)
	{
		global $db, $table_prefix;

		foreach ($tables as $table => $fields)
		{
			$result = $db->sql_query_limit('SELECT ' . implode(', ', array_keys($fields['columns'])) . ' FROM ' . self::getTableName($table), 1);

			if ($db->sql_error_triggered)
			{
				$db->sql_error_triggered = false;
				self::importTable($table, $table_prefix . $table);
			}
			else
			{
				$db->sql_freeresult($result);
			}
		}
	}

	public static function performSchemaChanges($schemaChanges)
	{
		global $dbtools;
		if (empty($dbtools))
		{
			self::createDBTools();
		}

		$changes = array();
		foreach (self::$schemaChangesTypes as $friendlyType => $phpbbType)
		{
			if (isset($this->schemaChanges[$friendlyType]))
			{
				$changes[$phpbbType] = $this->schemaChanges[$friendlyType];
			}
		}
		$dbtools->perform_schema_changes($changes);
	}

	public static function insertData($data)
	{
		global $db;

		foreach ($data as $table => $data)
		{
			/** @todo Add error reporting. */
			$db->sql_multi_insert(self::getTableName($table), $data);
		}
	}

	public static function performSqlQueries($sqlQueries)
	{
		global $db;

		foreach ($sqlQueries as $sql)
		{
			if (!$db->sql_query($sql))
			{
				$error = $db->sql_error();
				throw new InstallationTools_SQLError($error['message'], $error['code'], $sql);
			}
		}
	}

	public static function addPermissions($permissionOptions)
	{
		// Setup $auth_admin class so we can add permission options
		global $phpbb_root_path, $phpEx;
		include($phpbb_root_path . 'includes/acp/auth.' . $phpEx);
		$auth_admin = new auth_admin();

		// Add permissions
		$auth_admin->acl_add_option($permissionOptions); // This function automagically clears the ACL cache and phpbb_users.user_permissions.
	}

	public static function importTable($name, $realName)
	{
		global $db, $dbms, $table_prefix;

		$availableDbms = get_available_dbms($dbms);
		$dbmsData = $availableDbms[$dbms];

		// If mysql is chosen, we need to adjust the schema filename slightly to reflect the correct version.
		if ($dbms == 'mysql')
		{
			if (version_compare($db->mysql_version, '4.1.3', '>='))
			{
				$dbmsData['SCHEMA'] .= '_41';
			}
			else
			{
				$dbmsData['SCHEMA'] .= '_40';
			}
		}

		// Ok we have the db info go ahead and read in the relevant schema
		// and work on building the table
		$schemaFile = 'schemas/' . $name . '/' . $dbmsData['SCHEMA'] . '_schema.sql';

		// How should we treat this schema?
		$removeRemarks = $dbmsData['COMMENTS'];
		$delimiter = $dbmsData['DELIM'];

		$sqlQueries = @file_get_contents($schemaFile);

		$sqlQueries = str_replace('phpbb_' . $name, $realName, $sqlQueries);

		$removeRemarks($sqlQueries);

		$sqlQueries = split_sql_file($sqlQueries, $delimiter);

		foreach ($sqlQueries as $sql)
		{
			//$sql = trim(str_replace('|', ';', $sql));
			if (!$db->sql_query($sql))
			{
				$error = $db->sql_error();
				throw new InstallationTools_SQLError($error['message'], $error['code'], $sql);
			}
		}
	}
}

/**
* SQL Error exception
*
* @package Chat
**/
class InstallationTools_SQLError extends Exception
{
	public $sql;

	public function __construct($message, $code, $sql)
	{
		$this->sql = $sql;
		parent::__construct($message, $code);
	}
}